import { run } from "../validators"
import { currency } from "./currency"

describe('mapping objects and using as parsers', () => {
  test('parse / valid', async () => {
    expect(
      await run(currency.parse, {
        amount: '188.00e0',
        currency: 'USD'
      })
    ).toEqual([
      [],
      '$188',
      1
    ])
  })

  test('parse / invalid', async () => {
    expect(
      await run(currency.parse, {
        amount: '188.00',
        currency: 'DKK'
      })
    ).toEqual([
      ['currency must be one of USD or SEK'],
      '$188',
      0.5
    ])
  })

  test('parse / both invalid', async () => {
    expect(
      await run(currency.parse, { amount: '188k', currency: 'DKK' })
    ).toEqual([
      [
        'amount must be a number',
        'currency must be one of USD or SEK'
      ],
      '$0',
      0
    ])
  })

  test('unparse / ok', async () => {
    expect(
      await run(currency.unparse, "$0")
    ).toEqual([
      [],
      { amount: '0', currency: 'USD' },
      1
    ])
  })

  test('unparse / ok 2', async () => {
    expect(
      await run(currency.unparse, "123 kr")
    ).toEqual([
      [],
      { amount: '123', currency: 'SEK' },
      1
    ])
  })
})
