import { object, string, str2date, str2num, exactly, either } from "../schema"
import { run } from "../validators"

describe('schema', () => {
  test('simple parse and serialize / ok', async () => {
    const schema = object({
      foo: string,
      bar: string
    })

    expect(await run(schema.parse, { foo: 'hi', bar: 'world' })).toEqual([
      [],
      { foo: 'hi', bar: 'world' },
      1
    ])

    expect(await run(schema.unparse, { foo: 'hi', bar: 'world' })).toEqual([
      [],
      { foo: 'hi', bar: 'world' },
      1
    ])
  })

  test('simple parse and serialize / invalid', async () => {
    const schema = object({
      foo: string,
      bar: string
    })

    expect(await run(schema.parse, { bar: 'world' } as any)).toEqual([
      ['foo must be a string'],
      { foo: '', bar: 'world' },
      0.5
    ])

    expect(await run(schema.unparse, { bar: 'world' } as any)).toEqual([
      ['foo must be a string'],
      { foo: '', bar: 'world' },
      0.5
    ])
  })
})

describe('object', () => {
  const movie = object({
    released: str2date,
    rating: str2num
  })

  const book = object({
    author: string,
    ISBN: exactly('ABCD1234')
  })

  const ouvre =
    either(book, movie)

  test('object with boolean logic / ok 1', async () => {
    expect(await run(ouvre.parse, {
      released: '1989-04-21',
      rating: '9.8'
    } as any)).toEqual([
      [],
      {
        released: new Date('1989-04-21'),
        rating: 9.8
      },
      1
    ])

    expect(await run(ouvre.unparse, {
      released: new Date('1989-04-21'),
      rating: 9.8
    } as any)).toEqual([
      [],
      {
        released: '1989-04-21',
        rating: '9.8'
      },
      1
    ])
  })

  test('object with boolean logic / ok 2', async () => {
    expect(await run(ouvre.unparse, {
      author: 'Stephen King',
      ISBN: 'ABCD1234'
    } as any)).toEqual([
      [],
      {
        author: 'Stephen King',
        ISBN: 'ABCD1234'
      },
      1
    ])
  })
})
